﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.BusinessData;
using Microsoft.BusinessData.SystemSpecific;
using Microsoft.BusinessData.MetadataModel;
using Microsoft.BusinessData.MetadataModel.Collections;

namespace ProductConnector.ProductModel
{
    public partial class CategoryService : IContextProperty
    {
        public IList<Category> ReadCategories()
        {
            AdventureworksBCSCatalog catalog = new AdventureworksBCSCatalog(GetConnectionInfo());

            var q = from c in catalog.Categories
                    orderby c.Name
                    select c;

            List<Category> categories = new List<Category>();

            foreach (var i in q)
            {
                categories.Add(
                    new Category()
                    {
                        CategoryID = i.CategoryID,
                        Name = i.Name,
                    });
            }

            return categories;

        }

        public Category ReadCategory(int categoryID)
        {
            AdventureworksBCSCatalog catalog = new AdventureworksBCSCatalog(GetConnectionInfo());

            var q = from c in catalog.Categories
                    where c.CategoryID == categoryID
                    select c;

            if (q.Count() == 1)
            {
                return new Category()
                {
                    CategoryID = q.First().CategoryID,
                    Name = q.First().Name
                };
            }
            else
            {
                return null;
            }


        }

        public Category CreateCategory(Category newCategory)
        {
            AdventureworksBCSCatalog catalog = new AdventureworksBCSCatalog(GetConnectionInfo());

            ProductConnector.Category category = new ProductConnector.Category();

            category.CategoryID = newCategory.CategoryID;
            category.Name = newCategory.Name;

            catalog.AddToCategories(category);
            catalog.SaveChanges();
            newCategory.CategoryID = category.CategoryID;
            return newCategory;

        }

        public void UpdateCategory(Category category)
        {
            AdventureworksBCSCatalog catalog = new AdventureworksBCSCatalog(GetConnectionInfo());

            ProductConnector.Category updatedCategory = catalog.Categories.First(c => c.CategoryID == category.CategoryID);
            updatedCategory.CategoryID = category.CategoryID;
            updatedCategory.Name = category.Name;
            catalog.SaveChanges();

        }

        public void DeleteCategory(int categoryID)
        {
            AdventureworksBCSCatalog catalog = new AdventureworksBCSCatalog(GetConnectionInfo());

            ProductConnector.Category category = catalog.Categories.First(c => c.CategoryID == categoryID);
            catalog.DeleteObject(category);
            catalog.SaveChanges();

        }

        public IList<Product> CategoryToProduct(int categoryID)
        {
            AdventureworksBCSCatalog catalog = new AdventureworksBCSCatalog(GetConnectionInfo());

            var q = from p in catalog.Products
                    orderby p.Name
                    where p.CategoryID == categoryID
                    select p;

            List<Product> products = new List<Product>();

            foreach (var i in q)
            {
                products.Add(
                    new Product()
                    {
                        ProductID = i.ProductID,
                        CategoryID = i.CategoryID,
                        Name = i.Name,
                        Number = i.Number,
                        Color = i.Color,
                        Description = i.Description
                    });
            }

            return products;
        }

        internal string GetConnectionInfo()
        {
            INamedPropertyDictionary props =
              this.LobSystemInstance.GetProperties();
            if (props.ContainsKey("AdventureworksCatalog"))
                return props["AdventureworksCatalog"].ToString();
            else
                return string.Empty;
        }
        public Microsoft.BusinessData.Runtime.IExecutionContext
               ExecutionContext
        {
            get;
            set;
        }
        public Microsoft.BusinessData.MetadataModel.ILobSystemInstance
               LobSystemInstance
        {
            get;
            set;
        }
        public Microsoft.BusinessData.MetadataModel.IMethodInstance
               MethodInstance
        {
            get;
            set;
        }


    }
}
